"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var sketch_1 = require("./sketch");
var symbols_1 = require("../symbols");
var removal_reason_1 = require("../removal-reason");
var cache_node_1 = require("../cache-node");
var abstract_1 = require("../abstract");
var percentInMain = 0.99;
var percentProtected = 0.8;
var percentOverflow = 0.01;
var DATA = Symbol('boundedData');
/**
 * Bounded cache implementation using W-TinyLFU to keep track of data.
 *
 * See https://arxiv.org/pdf/1512.00727.pdf for details about TinyLFU and
 * the W-TinyLFU optimization.
 */
var BoundedCache = /** @class */ (function (_super) {
    __extends(BoundedCache, _super);
    function BoundedCache(options) {
        var _this = _super.call(this) || this;
        var maxMain = Math.floor(percentInMain * options.maxSize);
        /*
         * For weighted caches use an initial sketch size of 256. It will
         * grow when the size of the cache approaches that size.
         *
         * Otherwise set it to a minimum of 128 or the maximum requested size
         * of the graph.
         */
        var sketchWidth = options.weigher ? 256 : Math.max(options.maxSize, 128);
        _this[DATA] = {
            maxSize: options.weigher ? -1 : options.maxSize,
            removalListener: options.removalListener || null,
            weigher: options.weigher || null,
            weightedMaxSize: options.maxSize,
            weightedSize: 0,
            sketch: sketch_1.CountMinSketch.uint8(sketchWidth, 4),
            sketchGrowLimit: sketchWidth,
            values: new Map(),
            // Tracking of the window cache, around 1% of the total cache
            window: {
                head: new BoundedNode(null, null),
                size: 0,
                maxSize: options.maxSize - maxMain
            },
            // SLRU protected segment, 80% * 99% of the total cache
            protected: {
                head: new BoundedNode(null, null),
                size: 0,
                maxSize: Math.floor(maxMain * percentProtected)
            },
            // SLRU probation segment, 20% * 99% of the total cache
            probation: {
                head: new BoundedNode(null, null),
            },
            // Timeout used to schedule evictions
            evictionTimeout: 0,
            // The maximum size we can temporarily be grow before an eviction is forced
            forceEvictionLimit: options.maxSize + Math.max(Math.floor(options.maxSize * percentOverflow), 5),
            // The time to wait before an eviction is triggered by a set
            evictionInterval: 5000
        };
        return _this;
    }
    Object.defineProperty(BoundedCache.prototype, "maxSize", {
        /**
         * Get the maximum size this cache can be.
         */
        get: function () {
            return this[DATA].maxSize;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BoundedCache.prototype, "size", {
        /**
         * Get the current size of the cache.
         */
        get: function () {
            return this[DATA].values.size;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BoundedCache.prototype, "weightedSize", {
        /**
         * Get the weighted size of all items in the cache.
         */
        get: function () {
            return this[DATA].weightedSize;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Cache and associate a value with the given key.
     */
    BoundedCache.prototype.set = function (key, value) {
        var _this = this;
        var data = this[DATA];
        var old = data.values.get(key);
        // Create a node and add it to the backing map
        var node = new BoundedNode(key, value);
        data.values.set(key, node);
        if (data.weigher) {
            node.weight = data.weigher(key, value);
        }
        // Update our weight
        data.weightedSize += node.weight;
        if (old) {
            // Remove the old node
            old.remove();
            // Ajudst weight
            data.weightedSize -= old.weight;
            // Update weights of where the node belonged
            switch (old.location) {
                case 1 /* PROTECTED */:
                    // Node was protected, reduce the size
                    data.protected.size -= old.weight;
                    break;
                case 0 /* WINDOW */:
                    // Node was in window, reduce window size
                    data.window.size -= old.weight;
                    break;
            }
        }
        // Check if we reached the grow limit of the sketch
        if (data.weigher && data.values.size >= data.sketchGrowLimit) {
            var sketchWidth = data.values.size * 2;
            data.sketch = sketch_1.CountMinSketch.uint8(sketchWidth, 4);
            data.sketchGrowLimit = sketchWidth;
        }
        // Append the new node to the window space
        node.append(data.window.head);
        data.window.size += node.weight;
        // Register access to the key
        data.sketch.update(node.hashCode);
        // Schedule eviction
        if (data.weightedSize >= data.forceEvictionLimit) {
            this[symbols_1.EVICT]();
        }
        else if (!data.evictionTimeout) {
            data.evictionTimeout = setTimeout(function () { return _this[symbols_1.EVICT](); }, data.evictionInterval);
        }
        // Return the value we replaced
        if (old) {
            this[symbols_1.TRIGGER_REMOVE](key, old.value, removal_reason_1.RemovalReason.REPLACED);
            return old.value;
        }
        else {
            return null;
        }
    };
    /**
     * Get a previously cached value.
     */
    BoundedCache.prototype.getIfPresent = function (key) {
        var data = this[DATA];
        var node = data.values.get(key);
        if (!node) {
            // This value does not exist in the cache
            return null;
        }
        // Register access to the key
        data.sketch.update(node.hashCode);
        switch (node.location) {
            case 0 /* WINDOW */:
                // In window cache, mark as most recently used
                node.move(data.window.head);
                break;
            case 2 /* PROBATION */:
                // In SLRU probation segment, move to protected
                node.location = 1 /* PROTECTED */;
                node.move(data.protected.head);
                // Plenty of room, keep track of the size
                data.protected.size += node.weight;
                while (data.protected.size > data.protected.maxSize) {
                    /*
                        * There is now too many nodes in the protected segment
                        * so demote the least recently used.
                        */
                    var lru = data.protected.head.next;
                    lru.location = 2 /* PROBATION */;
                    lru.move(data.probation.head);
                    data.protected.size -= lru.weight;
                }
                break;
            case 1 /* PROTECTED */:
                // SLRU protected segment, mark as most recently used
                node.move(data.protected.head);
                break;
        }
        return node.value;
    };
    BoundedCache.prototype.peek = function (key) {
        var data = this[DATA];
        var node = data.values.get(key);
        return node ? node.value : null;
    };
    /**
     * Delete any value associated with the given key from the cache.
     */
    BoundedCache.prototype.delete = function (key) {
        var _this = this;
        var data = this[DATA];
        var node = data.values.get(key);
        if (node) {
            // Remove the node from its current list
            node.remove();
            switch (node.location) {
                case 1 /* PROTECTED */:
                    // Node was protected, reduce the size
                    data.protected.size -= node.weight;
                    break;
                case 0 /* WINDOW */:
                    // Node was in window, reduce window size
                    data.window.size -= node.weight;
                    break;
            }
            // Reduce overall weight
            data.weightedSize -= node.weight;
            // Remove from main value storage
            data.values.delete(key);
            this[symbols_1.TRIGGER_REMOVE](key, node.value, removal_reason_1.RemovalReason.EXPLICIT);
            if (!data.evictionTimeout) {
                data.evictionTimeout = setTimeout(function () { return _this[symbols_1.EVICT](); }, data.evictionInterval);
            }
            return node.value;
        }
        return null;
    };
    /**
     * Check if a certain value exists in the cache.
     */
    BoundedCache.prototype.has = function (key) {
        var data = this[DATA];
        return data.values.has(key);
    };
    BoundedCache.prototype.clear = function () {
        var e_1, _a;
        var data = this[DATA];
        var oldValues = data.values;
        data.values = new Map();
        try {
            for (var oldValues_1 = __values(oldValues), oldValues_1_1 = oldValues_1.next(); !oldValues_1_1.done; oldValues_1_1 = oldValues_1.next()) {
                var _b = __read(oldValues_1_1.value, 2), key = _b[0], node = _b[1];
                this[symbols_1.TRIGGER_REMOVE](key, node.value, removal_reason_1.RemovalReason.EXPLICIT);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (oldValues_1_1 && !oldValues_1_1.done && (_a = oldValues_1.return)) _a.call(oldValues_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        data.weightedSize = 0;
        data.window.head.remove();
        data.window.size = 0;
        data.probation.head.remove();
        data.protected.head.remove();
        data.protected.size = 0;
        if (data.evictionTimeout) {
            clearTimeout(data.evictionTimeout);
            data.evictionTimeout = null;
        }
    };
    BoundedCache.prototype.keys = function () {
        this[symbols_1.EVICT]();
        return Array.from(this[DATA].values.keys());
    };
    BoundedCache.prototype.cleanUp = function () {
        this[symbols_1.EVICT]();
    };
    Object.defineProperty(BoundedCache.prototype, "metrics", {
        get: function () {
            throw new Error('Metrics are not supported by this cache');
        },
        enumerable: true,
        configurable: true
    });
    BoundedCache.prototype[symbols_1.TRIGGER_REMOVE] = function (key, value, cause) {
        var data = this[DATA];
        // Trigger any extended remove listeners
        var onRemove = this[symbols_1.ON_REMOVE];
        if (onRemove) {
            onRemove(key, value, cause);
        }
        // Trigger the removal listener
        if (data.removalListener) {
            data.removalListener(key, value, cause);
        }
    };
    BoundedCache.prototype[symbols_1.EVICT] = function () {
        var data = this[DATA];
        /*
        * Evict the least recently used item in the window space to the
        * probation segment until we are below the maximum size.
        */
        var evictedToProbation = 0;
        while (data.window.size > data.window.maxSize) {
            var first = data.window.head.next;
            first.move(data.probation.head);
            first.location = 2 /* PROBATION */;
            data.window.size -= first.weight;
            evictedToProbation++;
        }
        /*
         * Evict items for real until we are below our maximum size.
         */
        while (data.weightedSize > data.weightedMaxSize) {
            var probation = data.probation.head.next;
            //const evicted = evictedToProbation == 0 ? null : data.probation.head.previous;
            var evicted = evictedToProbation === 0 ? data.probation.head : data.probation.head.previous;
            var hasProbation = probation !== data.probation.head;
            var hasEvicted = evicted !== data.probation.head;
            var toRemove = void 0;
            if (!hasProbation && !hasEvicted) {
                // TODO: Probation queue is empty, how is this handled?
                break;
            }
            else if (!hasEvicted) {
                toRemove = probation;
            }
            else if (!hasProbation) {
                toRemove = evicted;
                evictedToProbation--;
            }
            else {
                // Estimate how often the two items have been accessed
                var freqEvicted = data.sketch.estimate(evicted.hashCode);
                var freqProbation = data.sketch.estimate(probation.hashCode);
                if (freqEvicted > freqProbation) {
                    toRemove = probation;
                }
                else {
                    toRemove = evicted;
                }
                evictedToProbation--;
            }
            if (toRemove.key === null) {
                throw new Error('Cache issue, problem with removal');
            }
            data.values.delete(toRemove.key);
            toRemove.remove();
            data.weightedSize -= toRemove.weight;
            this[symbols_1.TRIGGER_REMOVE](toRemove.key, toRemove.value, removal_reason_1.RemovalReason.SIZE);
        }
        // Trigger the onEvict listener if one exists
        var onEvict = this[symbols_1.ON_EVICT];
        if (onEvict) {
            onEvict();
        }
        if (data.evictionTimeout) {
            clearTimeout(data.evictionTimeout);
            data.evictionTimeout = null;
        }
    };
    return BoundedCache;
}(abstract_1.AbstractCache));
exports.BoundedCache = BoundedCache;
/**
 * Node in a double-linked list.
 */
var BoundedNode = /** @class */ (function (_super) {
    __extends(BoundedNode, _super);
    function BoundedNode(key, value) {
        var _this = _super.call(this, key, value) || this;
        _this.hashCode = key == null ? 0 : sketch_1.CountMinSketch.hash(key);
        _this.weight = 1;
        _this.location = 0 /* WINDOW */;
        return _this;
    }
    return BoundedNode;
}(cache_node_1.CacheNode));
//# sourceMappingURL=index.js.map