"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_1 = require("../abstract");
var removal_reason_1 = require("../removal-reason");
var symbols_1 = require("../symbols");
var DATA = Symbol('boundlessData');
var EVICTION_DELAY = 5000;
/**
 * Boundless cache.
 */
var BoundlessCache = /** @class */ (function (_super) {
    __extends(BoundlessCache, _super);
    function BoundlessCache(options) {
        var _this = _super.call(this) || this;
        _this[DATA] = {
            values: new Map(),
            removalListener: options.removalListener || null,
            evictionTimeout: null
        };
        return _this;
    }
    Object.defineProperty(BoundlessCache.prototype, "maxSize", {
        /**
         * Get the maximum size this cache can be.
         */
        get: function () {
            return -1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BoundlessCache.prototype, "size", {
        /**
         * Get the current size of the cache.
         */
        get: function () {
            return this[DATA].values.size;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BoundlessCache.prototype, "weightedSize", {
        get: function () {
            return this.size;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Cache and associate a value with the given key.
     */
    BoundlessCache.prototype.set = function (key, value) {
        var _this = this;
        var data = this[DATA];
        var old = data.values.get(key);
        // Update with the new value
        data.values.set(key, value);
        // Schedule an eviction
        if (!data.evictionTimeout) {
            data.evictionTimeout = setTimeout(function () { return _this[symbols_1.EVICT](); }, EVICTION_DELAY);
        }
        // Return the value we replaced
        if (old !== undefined) {
            this[symbols_1.TRIGGER_REMOVE](key, old, removal_reason_1.RemovalReason.REPLACED);
            return old;
        }
        else {
            return null;
        }
    };
    /**
     * Get a value from this cache if it has been previously cached.
     */
    BoundlessCache.prototype.getIfPresent = function (key) {
        var data = this[DATA];
        var value = data.values.get(key);
        return value === undefined ? null : value;
    };
    BoundlessCache.prototype.peek = function (key) {
        var data = this[DATA];
        var value = data.values.get(key);
        return value === undefined ? null : value;
    };
    /**
     * Delete any value associated with the given key from the cache.
     */
    BoundlessCache.prototype.delete = function (key) {
        var _this = this;
        var data = this[DATA];
        var old = data.values.get(key);
        data.values.delete(key);
        if (old !== undefined) {
            // Trigger removal events
            this[symbols_1.TRIGGER_REMOVE](key, old, removal_reason_1.RemovalReason.EXPLICIT);
            // Queue an eviction event if one is not set
            if (!data.evictionTimeout) {
                data.evictionTimeout = setTimeout(function () { return _this[symbols_1.EVICT](); }, EVICTION_DELAY);
            }
            return old;
        }
        else {
            return null;
        }
    };
    /**
     * Check if a certain value exists in the cache.
     */
    BoundlessCache.prototype.has = function (key) {
        var data = this[DATA];
        return data.values.has(key);
    };
    /**
     * Clear all of the cached data.
     */
    BoundlessCache.prototype.clear = function () {
        var e_1, _a;
        var data = this[DATA];
        var oldValues = data.values;
        // Simply replace the value map new data
        data.values = new Map();
        try {
            // Trigger removal events for all of the content in the cache
            for (var _b = __values(oldValues.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), key = _d[0], value = _d[1];
                this[symbols_1.TRIGGER_REMOVE](key, value, removal_reason_1.RemovalReason.EXPLICIT);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * Get all of the keys currently in the cache.
     */
    BoundlessCache.prototype.keys = function () {
        this[symbols_1.EVICT]();
        return Array.from(this[DATA].values.keys());
    };
    /**
     * Clean up.
     */
    BoundlessCache.prototype.cleanUp = function () {
        // Simply request eviction so extra layers can handle this
        this[symbols_1.EVICT]();
    };
    Object.defineProperty(BoundlessCache.prototype, "metrics", {
        get: function () {
            throw new Error('Metrics are not supported by this cache');
        },
        enumerable: true,
        configurable: true
    });
    BoundlessCache.prototype[symbols_1.TRIGGER_REMOVE] = function (key, value, reason) {
        var data = this[DATA];
        // Trigger any extended remove listeners
        var onRemove = this[symbols_1.ON_REMOVE];
        if (onRemove) {
            onRemove(key, value, reason);
        }
        if (data.removalListener) {
            data.removalListener(key, value, reason);
        }
    };
    BoundlessCache.prototype[symbols_1.EVICT] = function () {
        // Trigger the onEvict listener if one exists
        var onEvict = this[symbols_1.ON_EVICT];
        if (onEvict) {
            onEvict();
        }
        var data = this[DATA];
        if (data.evictionTimeout) {
            clearTimeout(data.evictionTimeout);
            data.evictionTimeout = null;
        }
    };
    return BoundlessCache;
}(abstract_1.AbstractCache));
exports.BoundlessCache = BoundlessCache;
//# sourceMappingURL=index.js.map