"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_1 = require("../abstract");
var removal_reason_1 = require("../removal-reason");
var timer_wheel_1 = require("./timer-wheel");
var symbols_1 = require("../symbols");
var DATA = Symbol('expirationData');
/**
 * Wrapper for another cache that provides evictions of times based on timers.
 *
 * Currently supports expiration based on maximum age.
 */
var ExpirationCache = /** @class */ (function (_super) {
    __extends(ExpirationCache, _super);
    function ExpirationCache(options) {
        var _this = _super.call(this) || this;
        _this[symbols_1.PARENT] = options.parent;
        _this[DATA] = {
            maxWriteAge: options.maxWriteAge,
            maxNoReadAge: options.maxNoReadAge,
            removalListener: options.removalListener || null,
            timerWheel: new timer_wheel_1.TimerWheel(function (keys) {
                var e_1, _a;
                try {
                    for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                        var key = keys_1_1.value;
                        _this.delete(key);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            })
        };
        // Custom onRemove handler for the parent cache
        _this[symbols_1.PARENT][symbols_1.ON_REMOVE] = function (key, node, reason) {
            if (node.isExpired()) {
                reason = removal_reason_1.RemovalReason.EXPIRED;
            }
            _this[DATA].timerWheel.deschedule(node);
            _this[symbols_1.TRIGGER_REMOVE](key, node.value, reason);
        };
        // Custom eviction behaviour to advance the wheel
        _this[symbols_1.PARENT][symbols_1.ON_EVICT] = function () { return _this[DATA].timerWheel.advance(); };
        return _this;
    }
    Object.defineProperty(ExpirationCache.prototype, "maxSize", {
        get: function () {
            return this[symbols_1.PARENT].maxSize;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ExpirationCache.prototype, "size", {
        get: function () {
            return this[symbols_1.PARENT].size;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ExpirationCache.prototype, "weightedSize", {
        get: function () {
            return this[symbols_1.PARENT].weightedSize;
        },
        enumerable: true,
        configurable: true
    });
    ExpirationCache.prototype.set = function (key, value) {
        var data = this[DATA];
        var timerWheel = data.timerWheel;
        var node = timerWheel.node(key, value);
        var age = null;
        if (data.maxWriteAge) {
            age = data.maxWriteAge(key, value) || 0;
        }
        else if (data.maxNoReadAge) {
            age = data.maxNoReadAge(key, value) || 0;
        }
        if (age !== null && !data.timerWheel.schedule(node, age)) {
            // Age was not accepted by wheel, delete any previous value
            return this.delete(key);
        }
        try {
            var replaced = this[symbols_1.PARENT].set(key, node);
            return replaced ? replaced.value : null;
        }
        catch (ex) {
            timerWheel.deschedule(node);
            throw ex;
        }
    };
    ExpirationCache.prototype.getIfPresent = function (key) {
        var node = this[symbols_1.PARENT].getIfPresent(key);
        if (node) {
            if (node.isExpired()) {
                // Check if the node is expired and return null if so
                return null;
            }
            // Reschedule if we have a maximum age between reads
            var data = this[DATA];
            if (data.maxNoReadAge) {
                var age = data.maxNoReadAge(key, node.value);
                if (!data.timerWheel.schedule(node, age)) {
                    // Age was not accepted by wheel, expire it directly
                    this.delete(key);
                }
            }
            return node.value;
        }
        return null;
    };
    ExpirationCache.prototype.peek = function (key) {
        var node = this[symbols_1.PARENT].peek(key);
        return node && !node.isExpired() ? node.value : null;
    };
    ExpirationCache.prototype.has = function (key) {
        var node = this[symbols_1.PARENT].peek(key);
        return (node && !node.isExpired()) || false;
    };
    ExpirationCache.prototype.delete = function (key) {
        var node = this[symbols_1.PARENT].delete(key);
        return node ? node.value : null;
    };
    ExpirationCache.prototype.clear = function () {
        return this[symbols_1.PARENT].clear();
    };
    ExpirationCache.prototype.keys = function () {
        return this[symbols_1.PARENT].keys();
    };
    ExpirationCache.prototype.cleanUp = function () {
        return this[symbols_1.PARENT].cleanUp();
    };
    Object.defineProperty(ExpirationCache.prototype, "metrics", {
        get: function () {
            return this[symbols_1.PARENT].metrics;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ExpirationCache.prototype, symbols_1.ON_EVICT, {
        get: function () {
            return this[symbols_1.PARENT][symbols_1.ON_EVICT];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ExpirationCache.prototype, symbols_1.ON_EVICT, {
        set: function (listener) {
            this[symbols_1.PARENT][symbols_1.ON_EVICT] = listener;
        },
        enumerable: true,
        configurable: true
    });
    ExpirationCache.prototype[symbols_1.TRIGGER_REMOVE] = function (key, value, reason) {
        // Trigger any extended remove listeners
        var onRemove = this[symbols_1.ON_REMOVE];
        if (onRemove) {
            onRemove(key, value, reason);
        }
        var data = this[DATA];
        // Trigger the removal listener
        if (data.removalListener) {
            data.removalListener(key, value, reason);
        }
    };
    return ExpirationCache;
}(abstract_1.AbstractCache));
exports.ExpirationCache = ExpirationCache;
//# sourceMappingURL=index.js.map