import { WrappedCache } from '../wrapped';
const METRICS = Symbol('metrics');
/**
 * Extension to a cache that tracks metrics about the size and hit rate of
 * a cache.
 */
export class MetricsCache extends WrappedCache {
    constructor(options) {
        super(options.parent, options.removalListener || null);
        this[METRICS] = {
            hits: 0,
            misses: 0,
            get hitRate() {
                const total = this.hits + this.misses;
                if (total === 0)
                    return 1.0;
                return this.hits / total;
            }
        };
    }
    get metrics() {
        return this[METRICS];
    }
    getIfPresent(key) {
        const result = super.getIfPresent(key);
        if (result === null) {
            this[METRICS].misses++;
        }
        else {
            this[METRICS].hits++;
        }
        return result;
    }
}
//# sourceMappingURL=index.js.map