import { AbstractCache } from './abstract';
import { ON_REMOVE, ON_EVICT, TRIGGER_REMOVE } from './symbols';
const PARENT = Symbol('parent');
const REMOVAL_LISTENER = Symbol('removalListener');
/**
 * Wrapper for another cache, used to extend that cache with new behavior,
 * like for loading things or collecting metrics.
 */
export class WrappedCache extends AbstractCache {
    constructor(parent, removalListener) {
        super();
        this[PARENT] = parent;
        this[REMOVAL_LISTENER] = removalListener;
        // Custom onRemove handler for the parent cache
        this[PARENT][ON_REMOVE] = this[TRIGGER_REMOVE].bind(this);
    }
    get maxSize() {
        return this[PARENT].maxSize;
    }
    get size() {
        return this[PARENT].size;
    }
    get weightedSize() {
        return this[PARENT].weightedSize;
    }
    set(key, value) {
        return this[PARENT].set(key, value);
    }
    getIfPresent(key) {
        return this[PARENT].getIfPresent(key);
    }
    peek(key) {
        return this[PARENT].peek(key);
    }
    has(key) {
        return this[PARENT].has(key);
    }
    delete(key) {
        return this[PARENT].delete(key);
    }
    clear() {
        return this[PARENT].clear();
    }
    keys() {
        return this[PARENT].keys();
    }
    cleanUp() {
        return this[PARENT].cleanUp();
    }
    get metrics() {
        return this[PARENT].metrics;
    }
    get [ON_EVICT]() {
        return this[PARENT][ON_EVICT];
    }
    set [ON_EVICT](listener) {
        this[PARENT][ON_EVICT] = listener;
    }
    [TRIGGER_REMOVE](key, value, reason) {
        // Trigger any extended remove listeners
        const onRemove = this[ON_REMOVE];
        if (onRemove) {
            onRemove(key, value, reason);
        }
        // Trigger the removal listener
        const listener = this[REMOVAL_LISTENER];
        if (listener) {
            listener(key, value, reason);
        }
    }
}
//# sourceMappingURL=wrapped.js.map