import { KeyType } from '../key-type';
import { Cache } from '../cache';
import { CacheSPI } from '../cache-spi';
import { AbstractCache } from '../abstract';
import { RemovalListener } from '../removal-listener';
import { Metrics } from '../metrics/metrics';
import { ON_REMOVE, ON_EVICT, TRIGGER_REMOVE, EVICT } from '../symbols';
declare const DATA: unique symbol;
/**
 * Options for a boundless cache.
 */
export interface BoundlessCacheOptions<K extends KeyType, V> {
    /**
     * Listener that triggers when a cached value is removed.
     */
    removalListener?: RemovalListener<K, V> | undefined | null;
}
/**
 * Boundless cache.
 */
export declare class BoundlessCache<K extends KeyType, V> extends AbstractCache<K, V> implements Cache<K, V>, CacheSPI<K, V> {
    private [DATA];
    [ON_REMOVE]?: RemovalListener<K, V>;
    [ON_EVICT]?: () => void;
    constructor(options: BoundlessCacheOptions<K, V>);
    /**
     * Get the maximum size this cache can be.
     */
    get maxSize(): number;
    /**
     * Get the current size of the cache.
     */
    get size(): number;
    get weightedSize(): number;
    /**
     * Cache and associate a value with the given key.
     */
    set(key: K, value: V): V | null;
    /**
     * Get a value from this cache if it has been previously cached.
     */
    getIfPresent(key: K): V | null;
    peek(key: K): V | null;
    /**
     * Delete any value associated with the given key from the cache.
     */
    delete(key: K): V | null;
    /**
     * Check if a certain value exists in the cache.
     */
    has(key: K): boolean;
    /**
     * Clear all of the cached data.
     */
    clear(): void;
    /**
     * Get all of the keys currently in the cache.
     */
    keys(): K[];
    /**
     * Clean up.
     */
    cleanUp(): void;
    get metrics(): Metrics;
    private [TRIGGER_REMOVE];
    private [EVICT];
}
export {};
