import { KeyType } from '../key-type';
import { Cache } from '../cache';
import { CommonCacheOptions } from '../common-options';
import { CacheSPI } from '../cache-spi';
import { AbstractCache } from '../abstract';
import { Metrics } from '../metrics/metrics';
import { Expirable } from './expirable';
import { RemovalListener } from '../removal-listener';
import { MaxAgeDecider } from './max-age-decider';
import { PARENT, ON_REMOVE, TRIGGER_REMOVE, ON_EVICT } from '../symbols';
declare const DATA: unique symbol;
/**
 * Options available for a loading cache.
 */
export interface ExpirationCacheOptions<K extends KeyType, V> extends CommonCacheOptions<K, V> {
    maxWriteAge?: MaxAgeDecider<K, V>;
    maxNoReadAge?: MaxAgeDecider<K, V>;
    parent: Cache<K, Expirable<V>>;
}
/**
 * Wrapper for another cache that provides evictions of times based on timers.
 *
 * Currently supports expiration based on maximum age.
 */
export declare class ExpirationCache<K extends KeyType, V> extends AbstractCache<K, V> implements CacheSPI<K, V> {
    private [DATA];
    private [PARENT];
    [ON_REMOVE]?: RemovalListener<K, V>;
    constructor(options: ExpirationCacheOptions<K, V>);
    get maxSize(): number;
    get size(): number;
    get weightedSize(): number;
    set(key: K, value: V): V | null;
    getIfPresent(key: K): V | null;
    peek(key: K): V | null;
    has(key: K): boolean;
    delete(key: K): V | null;
    clear(): void;
    keys(): K[];
    cleanUp(): void;
    get metrics(): Metrics;
    get [ON_EVICT](): (() => void) | undefined;
    set [ON_EVICT](listener: (() => void) | undefined);
    private [TRIGGER_REMOVE];
}
export {};
