import { KeyType } from './key-type';
import { Cache } from './cache';
import { CacheSPI } from './cache-spi';
import { AbstractCache } from './abstract';
import { Metrics } from './metrics/metrics';
import { RemovalListener } from './removal-listener';
import { ON_REMOVE, ON_EVICT, TRIGGER_REMOVE } from './symbols';
declare const PARENT: unique symbol;
declare const REMOVAL_LISTENER: unique symbol;
/**
 * Wrapper for another cache, used to extend that cache with new behavior,
 * like for loading things or collecting metrics.
 */
export declare abstract class WrappedCache<K extends KeyType, V> extends AbstractCache<K, V> implements Cache<K, V>, CacheSPI<K, V> {
    private [PARENT];
    [ON_REMOVE]?: RemovalListener<K, V>;
    private [REMOVAL_LISTENER];
    constructor(parent: Cache<K, V> & CacheSPI<K, V>, removalListener: RemovalListener<K, V> | null);
    get maxSize(): number;
    get size(): number;
    get weightedSize(): number;
    set(key: K, value: V): V | null;
    getIfPresent(key: K): V | null;
    peek(key: K): V | null;
    has(key: K): boolean;
    delete(key: K): V | null;
    clear(): void;
    keys(): K[];
    cleanUp(): void;
    get metrics(): Metrics;
    get [ON_EVICT](): (() => void) | undefined;
    set [ON_EVICT](listener: (() => void) | undefined);
    private [TRIGGER_REMOVE];
}
export {};
